* Save/Load bytes to/from a channel
*
* pjwitte 2o2i+
* All code is my own work except as and where stated
*
* V0.01, pjw, 2018 Jan 29, simple all in one version
* V0.02. pjw, 2021 Jul 09, P'n'm version. New names
* V0.03, pjw, 2021 Oct 03, changed error checking
*
* Compatible with QDOS, Minerva and SMSQ/E
*
* Seems to work fine with odd addresses.. (Q-emulator w JS - QPC2 w SMSQ/E)
*
* NB: These commands only work with file channels! (This is where "device
* independence" breaks down..)
*


        section code

        xdef CH_SBYTES,CH_LBYTES

        xref getpars,chan_par,l2freta1

        include dev8_keys_sbasic
        include dev8_keys_qdos_io
        include dev8_keys_err

*
CH_SBYTES
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Usage: Save bytes from memory to channel
*
* er = CH_SBYTES(#ch; base, length)
*
* where
*       ch      is the S*BASIC number of a writeable channel
*       base    is the address where to save the data (odd or even ok)
*       length  is the expected length of the data (odd or even ok)
*
* Error returns in er: file/medium errors
* Parameter errors are "hard", ie they stop program execution
*

        moveq #iof.save,d7
        bsr.s parameters        no return on error
        bra.s do_it


*
CH_LBYTES
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Usage: Load from channel into memory
*
* er = CH_LBYTES(#ch; base, length)
*
* where
*       ch      is the S*BASIC number of an open non-console channel
*       base    is the address where to load the data (odd or even ok)
*       length  is the expected length of the data (odd or even ok)
*
* Error returns in er: NC, OR, file/medium errors (but no EOF!)
* Parameter errors are "hard", ie they stop program execution
*

        moveq #iof.load,d7
        bsr.s parameters        no return on error

*
do_it
        move.l d7,d0            load/save key
        trap #do.io

        move.l d0,d6
        bra l2freta1

*
parameters
        lea.l 3*8(a3),a4
        cmpa.l a4,a5            three parameters?
        bne.s no_ret_bp          not. Thats wrong

        bsr chan_par            ch in a0 (restores stack used)
        bne.s no_ret_err

        moveq #$26,d0           get two longs
        bsr getpars
        bne.s no_ret_err

        move.l 4(a6,a1.l),d2    d2 =  file length
        ble.s no_ret_or

        move.l (a6,a1.l),d0
        ble.s no_ret_or

        movea.l d0,a1           a1 -> base

        addq.l #2,sb_arthp(a6)  tidy, but leave room for float return
        moveq #forever,d3
        rts

*
no_ret_or
        moveq #err.orng,d0
        bra.s no_ret_err

*
no_ret_bp
        moveq #err.ipar,d0
*
no_ret_err
        addq.l #4,sp
        rts

*
        end
